<?php
namespace Ep\Hak;

use Ep\WordPress\Service\AbstractWordPressService;
use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Container\ContainerInterface;

/**
 * HAK integracija.
 *
 * @package Ep\Hak
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-hak
 */
class HakService extends AbstractWordPressService
{
	const META_KEY = '_hak_memberId';
	const COUPON_ID = PHP_INT_MAX - 12;
	const COUPON_CODE = 'hakcoupon';

	/** @var HttpClient */
	protected $httpClient;
	/** @var string */
	protected $memberId;
	/** @var array */
	protected $member;
	/** @var array */
	protected $memberKeys = [
		'memberId',
		'firstName',
		'lastName',
	];

	/**
	 * Konstruktor.
	 *
	 * @param ContainerInterface $container
	 * @param HttpClient $httpClient
	 */
	public function __construct(ContainerInterface $container, HttpClient $httpClient)
	{
		parent::__construct($container);
		$this->httpClient = $httpClient;
	}

	/**
	 *
	 *
	 * @return array
	 */
	public function getMember()
	{
		if (null === $this->member) {
			$user = wp_get_current_user();
			$this->member = [];

			if ($user->ID > 0) {
				foreach ($this->memberKeys as $key) {
					$this->member[$key] = get_user_meta($user->ID, sprintf('_hak_%s', $key), true);
				}
			} else if (isset($_SESSION['hak_data']) && is_array($_SESSION['hak_data'])) {
				$this->setMember($_SESSION['hak_data']);
			}
		}

		return $this->member;
	}

	/**
	 * @param array $data
	 */
	public function setMember(array $data)
	{
		foreach ($this->memberKeys as $key) {
			if (!array_key_exists($key, $data)) {
				throw new \RuntimeException(sprintf("HAK: Ključ '%s' nedostaje.", $key));
			}

			$this->member[$key] = $data[$key];
		}
	}

	/**
	 * Povezuje korisnikov račun s HAK-om.
	 *
	 * @param int $userId
	 *
	 * @return bool
	 */
	public function connect($userId = null)
	{
		if (!$this->validate()) {
			return false;
		}

		$member = $this->getMember();
		$user = wp_get_current_user();

		if ($user->ID > 0 || $userId) {
			foreach ($this->memberKeys as $key) {
				update_user_meta($userId ?: $user->ID, sprintf('_hak_%s', $key), $member[$key]);
			}
		} else {
			$_SESSION['hak_data'] = $member;
		}

		do_action('ep/hak/connect');
		return true;
	}

	/**
	 * Uklanja HAK s korisnikovog računa.
	 */
	public function disconnect()
	{
		$user = wp_get_current_user();

		if ($user->ID > 0) {
			foreach ($this->memberKeys as $key) {
				delete_user_meta($user->ID, sprintf('_hak_%s', $key));
			}
		}

		$this->member = null;
		unset($_SESSION['hak_data']);
		do_action('ep/hak/disconnect');
	}

	/**
	 * Vraća true ako je postavljeni član uistinu člank HAK-a.
	 *
	 * @return bool
	 */
	public function validate()
	{
		$config = $this->getConfig('hak');
		$member = $this->getMember();

		if (empty($member)) {
			return false;
		}

		$membership = $this->request($config['membership_endpoint'], [
			'first_name' => $member['firstName'],
			'last_name' => $member['lastName'],
			'member_id' => $member['memberId'],
		]);

		if (!is_array($membership) || true !== $membership['success']) {
			$this->member = null;
			unset($_SESSION['hak_data']);
			return false;
		}

		return true;
	}

	/**
	 * Upućuje zahtjev prema HAK api-ju.
	 *
	 * @param string $endpoint
	 * @param array $params
	 *
	 * @return array
	 */
	protected function request($endpoint, array $params = [])
	{
		try {
			$response = $this->httpClient->request('POST', $endpoint, [
				'json' => $params,
			]);

			$data = json_decode($response->getBody()->getContents(), true);

			if (null === $data) {
				throw new \RuntimeException(sprintf(
					"Došlo je do greške prilikom dekodiranja JSON-a: %s",
					json_last_error_msg()
				));
			}

			if (!is_array($data) || 0 === count($data)) {
				throw new \RuntimeException('Dobio sam neispravne podatke iz HAK-a.');
			}

			return $data[0];
		} catch (GuzzleException $e) {
			$this->log->err(sprintf(
				"Došlo je do greške prilikom komunikacije s HAK-om: %s (%s)",
				$e->getMessage(),
				json_encode($params)
			));
			throw new \RuntimeException("Došlo je do greške prilikom komunikacije s HAK-om.");
		}
	}
}