<?php
namespace Ep\Hak\Service;

use Ep\Hak\HakService;
use Ep\WordPress\Service\AbstractWordPressService;
use WC_Cart;
use Zend\Form\FormInterface;

/**
 * Class CartService
 *
 * @package Ep\Hak\Service
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-hak
 */
class CartService extends AbstractWordPressService
{
	/** @var WC_Cart */
	protected $cart;

	public function attach(): void
	{
		$config = $this->getConfig('hak');

		if ('1' === $config['enabled']) {
			add_action('ep/hak/connect', [$this, 'applyDiscount']);
			add_action('ep/hak/disconnect', [$this, 'removeDiscount']);
			add_action('woocommerce_add_to_cart', [$this, 'onCartChanged']);

			add_action('ep/cart/after_cart', [$this, 'renderForm']);

			$this->handleForm();

			if (is_object(WC()->cart)) {
				WC()->cart->get_cart();
			}
		}
	}

	/**
	 * Vraća trenutnu košaricu.
	 *
	 * @return WC_Cart
	 */
	protected function getCart()
	{
		if (null === $this->cart) {
			$cart = WC()->cart;

			if ($cart instanceof WC_Cart) {
				$cart->get_cart();
				$this->cart = $cart;
			}
		}

		return $this->cart;
	}

	/**
	 * Poziva se kada god dođe do promjene u košarici.
	 */
	public function onCartChanged()
	{
		/** @var HakService $hak */
		$hak = $this->services->get('Hak');

		if ($hak->validate()) {
			$this->applyDiscount();
		} else {
			$this->removeDiscount();
		}
	}

	/**
	 * Vraća true ako je HAK popust primjenjen na košaricu, false inače.
	 *
	 * @return bool
	 */
	public function hasDiscount()
	{
		foreach ($this->getCart()->get_applied_coupons() as $coupon) {
			if ($coupon === HakService::COUPON_CODE) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Primjenjuje HAK popust na košaricu.
	 */
	public function applyDiscount()
	{
		if (!$this->hasDiscount()) {
			$this->getCart()->apply_coupon(HakService::COUPON_CODE);
		}
	}

	/**
	 * Uklanja HAK kupon iz košarice kada se korisnik odspoji od HAK-a.
	 */
	public function removeDiscount()
	{
		if ($this->hasDiscount()) {
			$this->getCart()->remove_coupon(HakService::COUPON_CODE);
			wc_add_notice('HAK popust uklonjen.', 'error');
		}
	}

	protected function handleForm()
	{
		if ('POST' === $_SERVER['REQUEST_METHOD'] && isset($_POST['hakMemberId'])) {
			/** @var FormInterface $form */
			$form = $this->services->get('hak-membership');
			$form->setData($_POST);

			if ($form->isValid()) {
				/** @var HakService $hak */
				$hak = $this->services->get('Hak');
				$data = $form->getData();
				$data['memberId'] = $data['hakMemberId'];

				$hak->setMember($data);

				if ($hak->validate()) {
					$hak->connect();
					wc_add_notice('Uspješno ste povezali vaš račun s HAK-om!', 'success');
				} else {
					wc_add_notice('Nevažeći broj članske iskaznice.', 'error');
				}
			} else {
				$this->removeDiscount();
			}
		}
	}

	/**
	 * Renderira formu ispod košarice.
	 */
	public function renderForm()
	{
		$config = $this->getConfig('hak');
		/** @var HakService $hak */
		$hak = $this->services->get('Hak');

		if ($hak->validate()) {
			return;
		}

		/** @var FormInterface $form */
		$form = $this->services->get('hak-membership');
		$form->setAttribute('class', 'ep-hak-form');
		$form->remove('disconnect');

		echo $this->getView()->render('hak/cart-form.twig', [
			'form' => $form,
			'discount' => $config['discountPercentage'],
		]);
	}
}